@@ -1,6 +1,6 @@
 /* NickServ core functions
  *
- * (C) 2003-2020 Anope Team
+ * (C) 2003-2022 Anope Team
  * Contact us at team@anope.org
  *
  * Please read COPYING and README for further details.
@@ -86,7 +86,7 @@ class NickServRelease : public User, public Timer
 
  public:
 	NickServRelease(Module *me, NickAlias *na, time_t delay) : User(na->nick, Config->GetModule("nickserv")->Get<const Anope::string>("enforceruser", "user"),
-		Config->GetModule("nickserv")->Get<const Anope::string>("enforcerhost", "services.localhost.net"), "", "", Me, "Services Enforcer", Anope::CurTime, "", IRCD->UID_Retrieve(), NULL), Timer(me, delay), nick(na->nick)
+		Config->GetModule("nickserv")->Get<const Anope::string>("enforcerhost", Me->GetName()), "", "", Me, "Services Enforcer", Anope::CurTime, "", IRCD->UID_Retrieve(), NULL), Timer(me, delay), nick(na->nick)
 	{
 		/* Erase the current release timer and use the new one */
 		Anope::map<NickServRelease *>::iterator nit = NickServReleases.find(this->nick);
@@ -495,14 +495,14 @@ class NickServCore : public Module, public NickServService
 				"after %d days if not used."), nickserv_expire / 86400);
 	}
 
-	void OnNickCoreCreate(NickCore *nc)
+	void OnNickCoreCreate(NickCore *nc) anope_override
 	{
 		/* Set default flags */
 		for (unsigned i = 0; i < defaults.size(); ++i)
 			nc->Extend<bool>(defaults[i].upper());
 	}
 
-	void OnUserQuit(User *u, const Anope::string &msg)
+	void OnUserQuit(User *u, const Anope::string &msg) anope_override
 	{
 		if (u->server && !u->server->GetQuitReason().empty() && Config->GetModule(this)->Get<bool>("hidenetsplitquit"))
 			return;
