@@ -1,6 +1,6 @@
 /* Initialization and related routines.
  *
- * (C) 2003-2020 Anope Team
+ * (C) 2003-2022 Anope Team
  * Contact us at team@anope.org
  *
  * Please read COPYING and README for further details.
@@ -106,9 +106,12 @@ void Anope::Fork()
 #ifndef _WIN32
 	kill(getppid(), SIGUSR2);
 
-	freopen("/dev/null", "r", stdin);
-	freopen("/dev/null", "w", stdout);
-	freopen("/dev/null", "w", stderr);
+	if (!freopen("/dev/null", "r", stdin))
+		Log() << "Unable to redirect stdin to /dev/null: " << Anope::LastError();
+	if (!freopen("/dev/null", "w", stdout))
+		Log() << "Unable to redirect stdout to /dev/null: " << Anope::LastError();
+	if (!freopen("/dev/null", "w", stderr))
+		Log() << "Unable to redirect stderr to /dev/null: " << Anope::LastError();
 
 	setpgid(0, 0);
 
@@ -263,7 +266,9 @@ static void setuidgid()
 		{
 			LogFile* lf = li.logfiles[j];
 
-			chown(lf->filename.c_str(), uid, gid);
+			errno = 0;
+			if (chown(lf->filename.c_str(), uid, gid) != 0)
+				Log() << "Unable to change the ownership of " << lf->filename << " to " << uid << "/" << gid << ": " << Anope::LastError();
 		}
 	}
 
@@ -308,7 +313,7 @@ void Anope::Init(int ac, char **av)
 		Log(LOG_TERMINAL) << "Anope IRC Services (https://www.anope.org/)";
 		Log(LOG_TERMINAL) << "Usage ./" << Anope::ServicesBin << " [options] ...";
 		Log(LOG_TERMINAL) << "-c, --config=filename.conf";
-		Log(LOG_TERMINAL) << "    --confdir=conf file direcory";
+		Log(LOG_TERMINAL) << "    --confdir=conf file directory";
 		Log(LOG_TERMINAL) << "    --dbdir=database directory";
 		Log(LOG_TERMINAL) << "-d, --debug[=level]";
 		Log(LOG_TERMINAL) << "-h, --help";
@@ -551,12 +556,6 @@ void Anope::Init(int ac, char **av)
 	EventReturn MOD_RESULT;
 	FOREACH_RESULT(OnLoadDatabase, MOD_RESULT, ());
 	static_cast<void>(MOD_RESULT);
-
-	/* Compatibility for databases without ids */
-	for (nickcore_map::iterator it = NickCoreList->begin(), it_end = NickCoreList->end(); it != it_end; ++it)
-		it->second->GetId();
-	/* End compatibility for databases without ids */
-
 	Log() << "Databases loaded";
 
 	FOREACH_MOD(OnPostInit, ());
